# -*- coding:utf-8 -*-

import logging


LOGGING_FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO, format=LOGGING_FORMAT)


from pathlib import Path
import shutil
import sys

from tqdm import tqdm

from .common import get_public_domain_book_ids, image_filename_to_book, read_facts


INPUT_DIRNAME = Path(sys.argv[1])
INPUT_UPSCALED_HIGH_CONFIDENCE_FILENAMES = sys.argv[2]
INPUT_OCR_ROOT_TESSERACT = Path(sys.argv[3])
INPUT_OCR_ROOT_GOOGLE = Path(sys.argv[4])
INPUT_OCR_ROOT_COMBINED = Path(sys.argv[5])
INPUT_HUMAN_JUDGEMENTS = Path(sys.argv[6])
INPUT_HUMAN_JUDGEMENTS_WITH_COLUMNS_FILENAMES = Path(sys.argv[7])
INPUT_HUMAN_JUDGEMENTS_WITHOUT_COLUMNS_FILENAMES = Path(sys.argv[8])
LANGUAGE_DETECTION_ANNOTATIONS = Path(sys.argv[9])
CSV_DATABASE_FILENAME = Path(sys.argv[10])
OUTPUT_ROOT = Path(sys.argv[11])

PUBLIC_DOMAIN_BOOK_IDS = set(get_public_domain_book_ids(CSV_DATABASE_FILENAME))
INPUT_UPSCALED_HIGH_CONFIDENCE_LIST = list(read_facts(INPUT_UPSCALED_HIGH_CONFIDENCE_FILENAMES))
INPUT_UPSCALED_HIGH_CONFIDENCE_DICT = dict(INPUT_UPSCALED_HIGH_CONFIDENCE_LIST)
INPUT_HUMAN_JUDGEMENTS_WITH_COLUMNS_DICT = set(dict(read_facts(INPUT_HUMAN_JUDGEMENTS_WITH_COLUMNS_FILENAMES)).values())
INPUT_HUMAN_JUDGEMENTS_WITHOUT_COLUMNS_DICT = set(dict(read_facts(INPUT_HUMAN_JUDGEMENTS_WITHOUT_COLUMNS_FILENAMES)).values())


def produce_language_identification_annotations(directory):
    logger = logging.getLogger('produce_language_identification_annotations')

    directory.mkdir()

    num_copied = 0
    num_all = 0

    filenames = INPUT_UPSCALED_HIGH_CONFIDENCE_LIST
    filenames = tqdm(
        filenames,
        'Producing human annotations for language detection in {}/'.format(
            directory,
        )
    )

    for downscaled_filename, upscaled_filename in filenames:
        num_all += 1

        basename = Path(upscaled_filename).parent / Path(upscaled_filename).stem
        book_id, page = image_filename_to_book(downscaled_filename)

        input_filename = LANGUAGE_DETECTION_ANNOTATIONS / f'{basename}.hocr'

        if input_filename.exists():
            num_copied += 1
            output_filename = directory / f'{book_id}' / f'{page}.hocr'
            output_filename.parent.mkdir(exist_ok=True)
            shutil.copy(input_filename, output_filename)

    logger.info(f'Copied annotations for {num_copied} pages out of all {num_all} pages')


def produce_ocr_and_layout_analysis_annotations(directory):
    logger = logging.getLogger('produce_ocr_and_layout_analysis_annotations')

    directory.mkdir()

    with_columns_directory = directory / 'with-columns'
    without_columns_directory = directory / 'without-columns'

    with_columns_directory.mkdir()
    without_columns_directory.mkdir()

    num_copied = 0
    num_copied_with_columns = 0
    num_copied_without_columns = 0
    num_all = 0

    filenames = INPUT_UPSCALED_HIGH_CONFIDENCE_LIST
    filenames = tqdm(
        filenames,
        'Producing human annotations for OCR and layout analysis in {}/'.format(
            directory,
        )
    )

    for downscaled_filename, upscaled_filename in filenames:
        num_all += 1

        if upscaled_filename not in INPUT_HUMAN_JUDGEMENTS_WITH_COLUMNS_DICT \
                and upscaled_filename not in INPUT_HUMAN_JUDGEMENTS_WITHOUT_COLUMNS_DICT:
            continue

        assert upscaled_filename not in INPUT_HUMAN_JUDGEMENTS_WITH_COLUMNS_DICT \
            or upscaled_filename not in INPUT_HUMAN_JUDGEMENTS_WITHOUT_COLUMNS_DICT

        num_copied += 1

        basename = Path(upscaled_filename).parent / Path(upscaled_filename).stem
        book_id, page = image_filename_to_book(downscaled_filename)

        input_filename = INPUT_HUMAN_JUDGEMENTS / f'{basename}.txt'

        if upscaled_filename in INPUT_HUMAN_JUDGEMENTS_WITH_COLUMNS_DICT:
            num_copied_with_columns += 1
            output_with_columns_filename = with_columns_directory / f'{book_id}' / f'{page}.txt'
            output_with_columns_filename.parent.mkdir(exist_ok=True)
            shutil.copy(input_filename, output_with_columns_filename)

        if upscaled_filename in INPUT_HUMAN_JUDGEMENTS_WITHOUT_COLUMNS_DICT:
            num_copied_without_columns += 1
            output_without_columns_filename = without_columns_directory / f'{book_id}' / f'{page}.txt'
            output_without_columns_filename.parent.mkdir(exist_ok=True)
            shutil.copy(input_filename, output_without_columns_filename)

    logger.info(
        f'Copied annotations for {num_copied} pages ({num_copied_with_columns} with columns and '
        f'{num_copied_without_columns} without columns] out of all {num_all} pages'
    )


def produce_scanned_images_and_ocr_texts(images_directory, ocr_texts_directory):
    logger = logging.getLogger('produce_scanned_images_and_ocr_texts')

    images_directory.mkdir()
    ocr_texts_directory.mkdir()

    num_copied_images = 0
    num_copied_texts_tesseract = 0
    num_copied_texts_google = 0
    num_copied_texts_combined = 0
    num_public_domain = 0
    num_all = 0

    filenames = INPUT_UPSCALED_HIGH_CONFIDENCE_LIST
    filenames = tqdm(
        filenames,
        'Producing a dataset of scanned images in {}/ and OCR texts in {}/'.format(
            images_directory,
            ocr_texts_directory,
        )
    )

    for downscaled_filename, upscaled_filename in filenames:
        num_all += 1

        basename = Path(upscaled_filename).parent / Path(upscaled_filename).stem

        book_id, page = image_filename_to_book(downscaled_filename)
        if book_id not in PUBLIC_DOMAIN_BOOK_IDS:
            continue

        num_public_domain += 1

        input_image_filename = INPUT_DIRNAME / upscaled_filename
        if input_image_filename.exists():
            num_copied_images += 1
            output_image_filename = images_directory / f'{book_id}' / f'{page}{input_image_filename.suffix}'
            output_image_filename.parent.mkdir(exist_ok=True)
            shutil.copy(input_image_filename, output_image_filename)

        input_text_tesseract_filename = INPUT_OCR_ROOT_TESSERACT / f'{basename}.hocr'
        if input_text_tesseract_filename.exists():
            num_copied_texts_tesseract += 1
            output_text_tesseract_filename = ocr_texts_directory / f'{book_id}' / f'{page}.hocr'
            output_text_tesseract_filename.parent.mkdir(exist_ok=True)
            shutil.copy(input_text_tesseract_filename, output_text_tesseract_filename)

        input_text_google_filename = INPUT_OCR_ROOT_GOOGLE / f'{basename}.json'
        if input_text_google_filename.exists():
            num_copied_texts_google += 1
            output_text_google_filename = ocr_texts_directory / f'{book_id}' / f'{page}.json'
            output_text_google_filename.parent.mkdir(exist_ok=True)
            shutil.copy(input_text_google_filename, output_text_google_filename)

        input_text_combined_filename = INPUT_OCR_ROOT_COMBINED / f'{basename}.txt'
        if input_text_combined_filename.exists():
            num_copied_texts_combined += 1
            output_text_combined_filename = ocr_texts_directory / f'{book_id}' / f'{page}.txt'
            output_text_combined_filename.parent.mkdir(exist_ok=True)
            shutil.copy(input_text_combined_filename, output_text_combined_filename)

    logger.info(
        f'Copied {num_copied_images} images, {num_copied_texts_tesseract} Tesseract OCR texts, '
        f'{num_copied_texts_google} Google OCR texts, and {num_copied_texts_combined} combined '
        f'OCR texts from {num_public_domain} public domain pages out of all {num_all} pages'
    )


def main(directory=OUTPUT_ROOT):
    directory.mkdir(exist_ok=True)

    produce_scanned_images_and_ocr_texts(directory / 'scanned-images', directory / 'ocr-texts')
    produce_ocr_and_layout_analysis_annotations(directory / 'annotations-ocr')
    produce_language_identification_annotations(directory / 'annotations-language-identification')


if __name__ == '__main__':
    main()
